
#############################################################
#
#   solar.cl
#
#   Version 2
#
#   2/5/1996 (MPE)
#   
#    Modified from Version 1 (16/10/95)
#
#    Modifies the solar spectra generated by solar_lab
#     (i.e. combined -from the wn#### spectra-, rescaled,
#      rebinned to linear dispesion in lambda, divided by the
#      instrumental profile and multiplied by Kurucz continuum):
#
#     - star volocity correction
#     - orbital-barycentric motion
#     - convolution with stellaar rotational profile
#     - smoothing to lower resolution
#
#     The solar spectra are sun_j sun_h and sun_k and must be
#      be stored in the directory specified by sp_path
#
#############################################################

procedure solar (outspec,band)

string outspec=""     { prompt="Output solar spectrum" }
string band=""	      { prompt="Infrared spectral band (J,H,K)"}


string instfw="6.0"   { prompt="Instrumental profile FWHM (Angs.)" }

string vstar="0"      { prompt="Ref. star helioc. velocity (km/s)" }
string ra="00:00:00"  { prompt="Ref. star R.A. (hours)" }
string dec="00:00:00" { prompt="Ref. star Decl. (degrees)" }
string year="1994"    { prompt="Year (yyyy)" }
string month="11"     { prompt="Month (mm)" }
string day="8"        { prompt="Day (dd)" }

string rtstar="2"     { prompt="Ref. star rotat. velocity (km/s)" }

string sp_path="./"   { prompt="solar spectra directory path" }

bool  plot=yes        { prompt="Plot the solar spectrum?"}

begin

	real vvstar,rra, ddec
	real sigma, lst, iinstfw,rrtstar
	real wwcen,dlL,c1,c2,dl,dlonL,dlonLm,g,pi,eps
	int  iyear,imonth,iday
	string ooutspec,filelist
	real   rmean1,rmean2,ratio
	string sp_path2,vstar2,air2,band2
    string dum1,dum2,dum3,dum4,dum5,dum6,svorb
	real vorb,cv

    real oldstartw,oldstep,newstartw,newstep,new_z


# Limb-darkening coefficient
	eps = 0.25

	pi = 3.1415927
    cv = 2.997925e+5

noao
astutil
onedspec
dataio
images

# Passes parameters

	ooutspec = outspec
    band2 = band
	sp_path2 = sp_path
	vstar2 = vstar
	iyear = int(year)
	imonth = int(month)
	iday = int(day)
	rra = real(ra)
	ddec = real(dec)
	vvstar = real(vstar)
	iinstfw = real(instfw)
	rrtstar = real(rtstar)


#  Check existance of the output image

	mkspec (ooutspec,"dummy",10,1,function=1,constant=0)

# Clean up
	del ("orbit.v", ver-, >& "dev$null" )
	del ("kernel", ver-, >& "dev$null" )
	del ("mean1.tmp", ver-, >& "dev$null" )
	del ("mean2.tmp", ver-, >& "dev$null" )
	imdel ("nosmth", ver-, >& "dev$null" )
	imdel ("broad", ver-, >& "dev$null" )
	imdel (ooutspec, ver-, >& "dev$null" )


# Calculates the orbital projected velocity

	rvcorrect (header+, year=iyear, month=imonth,\
        	day=iday, ut=0., observa="mmt", vobs=0.,\
		epoch=indef, dec=ddec, ra=rra, > "orbit.v")

    list="orbit.v"
    while(fscan(list,dum1,dum2,dum3,dum4,dum5,dum6,svorb) != EOF){
    }
      vorb = real(svorb)
#      print("vorb = "//vorb)

# Reads spectra and parameters according to the spectral band

	print(" Reading data...")
    if ((band2=="J")||(band2=="j")){
       imcopy(sp_path2//"sun_j","nosmth",ver-)
       lst = 0.0120609952136874
       wwcen = 12395
    }
    else if ((band2=="H")||(band2=="h")){
       imcopy(sp_path2//"sun_h","nosmth",ver-)
       lst = 0.020227000117302
       wwcen = 16360
     }
     else if ((band2=="K")||(band2=="k")){
       imcopy(sp_path2//"sun_k","nosmth",ver-)
       lst = 0.036836002022028
       wwcen = 22054
     }
     else {
       print("Unknown or not available spectral band: "//band2)
       return
     } 

#  shifts the spectrum according to vstar and vorb

    imgets("nosmth","CD1_1")
    oldstep = real(imgets.value)
    imgets("nosmth","CRVAL1")
    oldstartw = real(imgets.value)
    new_z = (vvstar-vorb)/cv 
    newstartw = oldstartw*(1+new_z)
    newstep = oldstep*(1+new_z)
    hedit("nosmth","CD1_1",newstep,add-,del-,ver-,show-,update+)
    hedit("nosmth","CDELT1",newstep,add-,del-,ver-,show-,update+)
    hedit("nosmth","CRVAL1",newstartw,add-,del-,ver-,show-,update+)


# Apply rotational velocity 

	if (rrtstar > 4.){
		print(" Convolving with the rotational profile...")
		print(0.0, > "kernel")
		dlL = wwcen*rrtstar/2.997925e+5
		c1 = 2.*(1.-eps)/(pi*dlL*(1.-eps/3.))
		c2 = eps/(2.*dlL*(1.-eps/3.))
		dl = -dlL+lst
		while(dl <= dlL){
		  dlonL = dl/dlL
		  dlonLm = 1.-dlonL*dlonL
		  g = c1*sqrt(dlonLm)+c2*dlonLm
		  print(g, >> "kernel")
		  dl = dl+lst
		  }
		print(0.0, >> "kernel")
		convolve("nosmth","broad",kernel="kernel",\
		  bilinear-,boundary="nearest")
		imstat("nosmth",fields="mean",\
		  lower=INDEF,upper=INDEF,format-,\
		  > "mean1.tmp")
		imstat("broad",fields="mean",\
		  lower=INDEF,upper=INDEF,format-,\
		  > "mean2.tmp")
		list="mean1.tmp"
		while(fscan(list,rmean1)!=EOF){
			}
		list="mean2.tmp"
		while(fscan(list,rmean2)!=EOF){
			}
		ratio = rmean1/rmean2
		imarith("broad","*",ratio,"broad")
		}
	else {
		imrename("nosmth","broad")
		}


# Smooths the spectrum
	
	print(" Smoothing the spectrum...")
	 sigma = iinstfw/(lst*2.35482)
	gauss("broad",ooutspec,sigma,ratio=1., theta=0., nsigma=4.,\
		bilinear=yes, boundary="nearest")

# plot
        if (plot){
	  splot(ooutspec,1,1,units="angstroms", options="auto wreset",\
		 cursor="dev$null")
          splot.cursor=""
        }

# Clean up

	del ("orbit.v", ver-, >& "dev$null" )
	del ("kernel", ver-, >& "dev$null" )
	del ("mean1.tmp", ver-, >& "dev$null" )
	del ("mean2.tmp", ver-, >& "dev$null" )
	imdel ("nosmth", ver-, >& "dev$null" )
	imdel ("broad", ver-, >& "dev$null" )

end
